# hypothesis 1b - how far participants agree what to call a particular sound vs which emotion it expresses
setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
library(ggplot2)
library (reshape2)
require(brms)
require(shinystan)

### Import data: a list of sounds with two entropy measures for each. The interesting question is whether callEntropy is smaller than emEntropy 
temp_sv = as.matrix (read.csv('table_simpl_sv.csv')[,c('sound','callEntropy','emEntropy')])
temp_en = as.matrix (read.csv('table_simpl_en.csv')[,c('sound','callEntropy','emEntropy')])
temp_ru = as.matrix (read.csv('table_simpl_ru.csv')[,c('sound','callEntropy','emEntropy')])
# average over three languages
dfEntropyPool = array (c(temp_sv,temp_en,temp_ru), dim=c(132,3,3))
dfEP = apply (dfEntropyPool[,2:3,], c(1,2), function(x) mean(as.numeric(x)))
dfEP = data.frame ('sound'=temp_sv[,1], 'callEntropy'=dfEP[,1], 'emEntropy'=dfEP[,2])
# write.csv (dfEP, 'dfEP.csv')



### Model for data averaged over 3 languages (~t-test for dependent samples)
a = melt(dfEP)
aggregate (value~variable, a, mean) # sanity check - basic numbers
ggplot(data=a, aes(x=variable, y=value))+geom_violin()+geom_point() # visual inspection of normality, variances etc

mod0 = brm (value~variable + (1|sound), data = a, n.warmup=100, n.iter=5000, n.chains=4, n.cluster=4, family='gaussian', ranef=F) # note the random intercept (1|sound) - b/c we have nesting by sound (samples are not independent)
plot (mod0)  # check for convergence of chains, etc
summary(mod0)  # take overall difference b/w callEntropy and emEntropy from this summary: difference between emEntropy and callEntropy is 9.6 [7.7, 11.6] 
# stancode (mod0)  # look at the actual model. If needed, priors etc can be changed manually. For ex., if variances look different in the two groups, add that. Or specify t- instead of normal distributions if there are obvious outliers. Etc - see Kruschke's book, for example.

coda0 = posterior_samples(mod0) # save MCMC chains for posterior prediction
colnames(coda0)
coda0$call_total = coda0[,1] # this vector contains credible values of callEntropy (averaged for all 3 languages). Take mean / median, quntiles or whatever you want
coda0$em_total = coda0[,1] + coda0[,2] # same for emEntropy



### Model for three languages as three groups
a_sv = melt(as.data.frame(temp_sv), id.vars='sound'); a_sv$lang = 'sv'
a_en = melt(as.data.frame(temp_en), id.vars='sound'); a_en$lang = 'en'
a_ru = melt(as.data.frame(temp_ru), id.vars='sound'); a_ru$lang = 'ru'
df = rbind(a_sv, a_en, a_ru)
df$lang = as.factor(df$lang); df$value = as.numeric(df$value)

aggregate (value~variable+lang, df, mean)  # Sanity check. Three groups, paired observations, so ~repeated-measures ANOVA. Visualized, check normality etc as needed (can also be seen in the final graph)
drop1(lm(value~variable*lang, df), test='F') # A quick check using frequentist statistics. Might allow interaction anyway, though it's weak
mod = brm (value~variable*lang + (1|sound), data = df, n.warmup=100, n.iter=5000, n.chains=4, n.cluster=4, family='gaussian', ranef=F) 
# plot (mod)
# summary(mod)  # here interaction looks needed (CI for variableemEntropy:langru excludes zero)

coda = posterior_samples(mod) # save chains
colnames(coda)

# posterior prediction
coda$call_en = coda[,1]
coda$em_en = coda[,1] + coda[,2]
coda$call_ru = coda[,1] + coda[,3]
coda$em_ru = coda[,1] + coda[,2] + coda[,3] + coda[,5]
coda$call_sv = coda[,1] + coda[,4]
coda$em_sv = coda[,1] + coda[,2] + coda[,4] + coda[,6]


### Plotting. Note we take medians of posterior distribution and 2.5% & 97.5% quantiles as 95% CI. coda contains MCMC chains for the model with 3 groups, coda0 for the averaged model
dfPlot = data.frame ('Language'=rep(c('English','Russian','Swedish','All'),each=2), 
      Measure = rep(c('Sound name','Emotion name'),4), 
      Entropy = c ( as.numeric(apply (coda[,10:15], 2, function(x)quantile(x, .5))), median(coda0$call_total), median(coda0$em_total)), 
      lwr = c (as.numeric(apply (coda[,10:15], 2, function(x)quantile(x, .025))), quantile(coda0$call_total,.025), quantile(coda0$em_total,.025)),
      upr = c(as.numeric(apply (coda[,10:15], 2, function(x)quantile(x, .975))), quantile(coda0$call_total,.975), quantile(coda0$em_total,.975)) )

# set factor levels for the desired order of presentation in the graph
dfPlot$Measure = factor(dfPlot$Measure, levels=c('Sound name', 'Emotion name'))
dfPlot$Language = factor(dfPlot$Language, levels=c('English','Swedish','Russian','All'))

# ...and plot
ggplot (dfPlot, aes(x=Language, y=Entropy, fill=Measure)) +
  geom_bar (position=position_dodge(.7), stat="identity", width=.7) + 
  geom_errorbar (aes(ymin=lwr, ymax=upr),
                 width=.2,                   
                 position=position_dodge(.7)) +
  geom_vline(xintercept = 3.5, colour="gray", linetype = "longdash") +
  scale_fill_manual(values=c('gray50','gray80'), name="") +
  xlab('') + ylab('Entropy, %') +
  theme_bw() + theme(legend.position='top')

ggsave ('pix/entropy.png', scale=.5) # save the graph


## plotting with violins
temp1 = aggregate (value~variable+sound, a, mean) # averaged 3 languages
temp1$variable = rep(c('Sound name','Emotion name'))
temp1$lang = 'All'

temp2 = aggregate (value~variable+lang+sound, df, mean) # 3 languages separately
temp2$variable = rep(c('Sound name','Emotion name'))
temp2$lang = rep(c('English','English','Russian','Russian','Swedish','Swedish'))

temp = rbind (temp1, temp2)
colnames(temp) = c('Measure','sound','Entropy','Language')
temp$Measure = factor (temp$Measure, levels=c('Sound name','Emotion name'))
temp$Language = factor (temp$Language, levels=c('English','Swedish','Russian','All'))

ggplot (dfPlot, aes(x=Language, y=Entropy, fill=Measure)) +
  geom_violin(data=temp, aes(x=Language, y=Entropy, fill=Measure), alpha=0.3, position=position_dodge(.7), col='white', show_guide=T) +
  geom_point(data=temp, aes(x=Language, y=Entropy), 
             alpha=.2, 
             position = position_jitterdodge(dodge.width=.7, jitter.width=.1),
             size=1, show_guide=FALSE) +
  geom_errorbar (aes(ymin=lwr, ymax=upr),
                 width=.2,                   
                 position=position_dodge(.7)) +
  geom_point(shape=21, size=2, position=position_dodge(.7), show_guide=FALSE) +
  geom_vline(xintercept = 3.5, colour="gray", linetype = "longdash") +
  scale_fill_manual(values=c('gray40','gray80'), name="") +
  xlab('') + ylab('Entropy, %') +
  theme_bw() + theme(legend.position='top')

ggsave ('pix/entropy_violins.png', width=9, height=6, scale=1.7, units='cm', dpi=300)


## correlation b/w callEntropy and emEntropy
cor (dfEP$callEntropy, dfEP$emEntropy)
summary(lm(callEntropy~emEntropy, dfEP))

mod = brm (callEntropy~emEntropy, data=dfEP, n.warmup=100, n.iter=5000, n.chains=4, n.cluster=4, family='gaussian') 
plot (mod)
summary(mod)
coda = posterior_samples(mod)
colnames(coda)
coda$cor = coda[,2] * sd(dfEP$emEntropy) / sd(dfEP$callEntropy)
quantile(coda$cor, probs=c(.025,.5,.975)) # Pearson's r with 95% CI


## Effect size: Cohen's d for entropy (total)
m1 = mean(dfEP$callEntropy)
m2 = mean(dfEP$emEntropy)
s1 = var(dfEP$callEntropy)
s2 = var(dfEP$emEntropy)
n1 = nrow(dfEP)
n2 = nrow(dfEP)
d = (m1-m2) / sqrt( ((n1-1)*s1 + (n2-1)*s2)/(n1+n2-2) ) # .76
r = cor (dfEP$callEntropy, dfEP$emEntropy) # .58
d4 = d / sqrt (1-r) # 1.19

# or: mean(dfEP$callEntropy-dfEP$emEntropy) / sd(dfEP$callEntropy-dfEP$emEntropy)
